function varargout = minLocalSearchFirstImproving(...
    evalfunc, ...               %
    xstart, ...
    sampleneighborhood, ...
    evalfuncparams, ...
    sampleneighborhoodparams, ...
    o)
%
% MINLOCALSEARCHFIRSTIMPROVING Uses local search with first-improving
% strategy to find minimum of objectiv function.
%
% Arguments:
%   evalfunc
%       handle of objective function
%   xstart
%       initial point
%   sampleneighborhood
%       handle of the perturbation operator
%   evalfuncparams
%       additional parameters of objective function
%   sampleneighborhoodparams
%       additional parameters of the perturbation operator
%   o
%       options:
%       o.MaxFunEvals - maximal number of objective function evaluations
%       o.StopFitness - the goal fitness, if the best-so-far fitness is
%           lower than o.StopFitness, the search stops
%       o.verbose - boolean; if it should print some information about
%           solution progress

% Check and prepare options, set default values if needed
if ~exist('o','var') | isempty(o), o = struct; end
if iscell(o), o = struct(o{:}); end
% if ~isfield(o,'TolX'), o.TolX = 1e-10; end
% if ~isfield(o,'TolFun'), o.TolFun = 1e-8; end
if ~isfield(o,'MaxFunEvals'), o.MaxFunEvals = 20000; end
% if ~isfield(o,'MaxIter'), o.MaxIter = inf; end
if ~isfield(o,'StopFitness'), o.StopFitness = 1e-6; end
if ~isfield(o,'verbose'), o.verbose = 0; end

% Prepare and evaluate inital point
x = xstart;
fx = feval(evalfunc, x, o, evalfuncparams{:});

% Initialize history
hist.evals = 0;
hist.x = xstart;
hist.fmin = fx;
hist.nonimpr = 0;

% The main cycle
iter = 0;
nonimpr = 0;
while (iter < o.MaxFunEvals) & ...
      (fx > o.StopFitness),
    
    % Increase iteration counter
    iter = iter + 1;
    
    % Generate new point using the perturbation operator
    xn = feval(sampleneighborhood, x, o, sampleneighborhoodparams{:});
    % Evaluate it
    fxn = feval(evalfunc, xn, o, evalfuncparams{:});
    
    % If the new point is better, go for it!
    if fxn < fx,
        x = xn;
        fx = fxn;
        hist.evals(end+1) = iter;
        hist.x(end+1,:) = x;
        hist.fmin(end+1) = fx;
        hist.nonimpr(end+1) = nonimpr;
        nonimpr = 0;
    else
        nonimpr = nonimpr + 1;
    end
    
end
    
% Finish statistics
hist.evals(end+1) = iter;
hist.x(end+1,:) = x;
hist.fmin(end+1) = fx;
hist.nonimpr(end+1) = nonimpr;

% Set outputs
varargout{1} = x;
if nargout > 1, varargout{2} = fx; end
if nargout > 2, varargout{3} = hist; end
    